-- Register plugin
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local Party = RW:NewModule("Party", "AceEvent-3.0", "AceTimer-3.0")
if not Party then return end

RW.Party = Party

-- Locals
local wipe = wipe
local GetNumRaidMembers, GetNumPartyMembers, UnitName, UnitClass = GetNumRaidMembers, GetNumPartyMembers, UnitName, UnitClass
local GetRaidRosterInfo, UnitIsPartyLeader = GetRaidRosterInfo, UnitIsPartyLeader
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")

local pName = UnitName("player")
local pClass = select(2, UnitClass("player"))

local Com

function Party:OnInitialize()
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PartyUpdate")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "PartyUpdate")
	
	self.group = {}
end

function Party:OnEnable()	
	self.curParty = "none"
	
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PartyUpdate")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "PartyUpdate")	
	
	self:ScheduleTimer("FirstUpdate", 3)
	
	Com = RW.Com
end

function Party:OnDisable()
	wipe(self.group)
end

function Party:FirstUpdate()
	self:PartyUpdate("PARTY_MEMBERS_CHANGED")
	self:PartyUpdate("RAID_ROSTER_UPDATE")
end

function Party:PartyUpdate(event)
	local partyType = self:GetPartyType()
	if (partyType == "raid" or partyType == "battleground") and event == "RAID_ROSTER_UPDATE" then
		if self.curParty ~= partyType then
			RW.Callbacks:Fire("PartyJoined", partyType)
			RW.Com:Send("IncVersionInfo", partyType, nil, RWVERSION, RWREVISION, RWRELEASETPYE)
		end
		for i = 1, GetNumRaidMembers() do
			local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i);
			if name and name ~= "Unknown" then
				if not self.group[name] and name ~= pName then
					RW.Callbacks:Fire("PartyPlayerJoined", name)
					if online then
						RW.Com:Send("ReqVersionInfo", "WHISPER", name)
					end
				end				
			end		
			if name then
				self.group[name] = self.group[name] or {}
				self.group[name].name = name
				self.group[name].class = fileName
				self.group[name].rank = rank
				self.group[name].subgroup = subgroup
				self.group[name].zone = zone
				self.group[name].id = "raid"..i
				self.group[name].updated = true
			end
		end
		
		for i, player in pairs(self.group) do
			if not player.updated then
				RW.Callbacks:Fire("PartyPlayerLeft", player.name)
				self.group[i] = nil
			else
				player.updated = nil
			end
		end
		
		self.curParty = partyType
	
	elseif partyType == "party" and event == "PARTY_MEMBERS_CHANGED" then
		if self.curParty ~= partyType then
			RW.Callbacks:Fire("PartyJoined", partyType)
			RW.Com:Send("IncVersionInfo", partyType, nil, RWVERSION, RWREVISION, RWRELEASETPYE)
		end
		for i = 0, GetNumPartyMembers() do
			local id = i == 0 and "player" or "party"..i
			local name = UnitName(id)
			if name and name ~= "Unknown" then
				if not self.group[name] and name ~= pName then
					RW.Callbacks:Fire("PartyPlayerJoined", name)
					if UnitIsConnected(id) then
						RW.Com:Send("ReqVersionInfo", "WHISPER", name)
					end
				end
			end
			if name then
				self.group[name] = self.group[name] or {}
				self.group[name].name = name
				self.group[name].class = select(2, UnitClass(id))
				self.group[name].rank = UnitIsPartyLeader(id) and 2 or 0
				self.group[name].id = id
				self.group[name].updated = true
			end
		end
		
		for i, player in pairs(self.group) do
			if not player.updated then
				RW.Callbacks:Fire("PartyPlayerLeft", player.name)
				self.group[i] = nil
			else
				player.updated = nil
			end
		end
		
		self.curParty = partyType
		
	elseif partyType == "none" then
		if self.curParty ~= partyType then
			RW.Callbacks:Fire("PartyLeft", self.curParty)
		end
		for i, player in pairs(self.group) do
			self.group[i] = nil
		end
		self.group[pName] = self.group[pName] or {}
		self.group[pName].name = pName
		self.group[pName].class = pClass
		self.group[pName].rank = 2
		self.group[pName].id = "player"
		self.group[pName].version = tonumber(RWVERSION)
		self.group[pName].revision = tonumber(RWREVISION)
		self.group[pName].releaseType = RWRELEASETPYE
		
		self.curParty = partyType
	end
end

--- Get the current party type
-- @return Current party type (raid, party, battleground)
function Party:GetPartyType()return ((select(2, IsInInstance()) == "pvp" and "battleground") or (select(2, IsInInstance()) == "arena" and "battleground") or (GetNumRaidMembers() > 0 and "raid") or (GetNumPartyMembers() > 0 and "party") or "none") end

--- Get current party type localized
-- @return Current party type in localized format.
function Party:GetPartyTypeLocalized()
	local partytype = Party:GetPartyType()
	if partytype == "none" then return null end
	return LB[partytype]
end

--- Get the current size of the party.
-- @return Current party size.
function Party:GetPartySize() return  math.max(GetNumRaidMembers(), GetNumPartyMembers() + 1) end

--- Check if player is in current party by name
-- @param	name	string	Name of the player to check for
-- @return True if the player is in the party, else nil.
function Party:InParty(name) if self.group[name] then return true end end

--- Check if player is currently master looter.
-- @param	name	string	Name of the player to check
-- @return True if the player is the current master looter, else nil.
function Party:IsML(name) return self.group[name] and self.group[name].rank == 2 end

--- Check if currently in a party.
-- @return True if in a party, else false.
function Party:InParty() return self.currParty ~= "" end

--- Check the current rank of a player.
-- @param	name	string	Name of the player to check.
-- @return Returns the rank of the player if currently in the same party, else nil.
function Party:GetRank(name) return self.group[name] and self.group[name].rank end

--- Get an interator for the current party
-- @return An iterator to iterate over the current party.
function Party:PartyIterator() return pairs(self.group) end

--- Get the number of player currently alive in the party.
-- @return The number of alive people.
function Party:GetNumAlive()
	local alive = 0
	for i, player in pairs(self.group) do
		alive = alive + (UnitIsDeadOrGhost(player.id) and 0 or 1)
	end
	return alive
end

--- Fimd a party member with current target matching the one passed in.
-- @param	target	string Name of the target to search for.
-- @return the uID of a party member targeting the target if found, else false.
function Party:GetPartyTargetId(target)
	if not target then return end
	for i, member in pairs(self.group) do
		if UnitName(member.id.."target") == target then
			return member.id.."target"
		end
	end
	return false
end
